// Add addon imports here, try used to prevent errors from crashing GDS
import "./image-display/addon.js"
import "./sequencer/addon.js"
import "./advanced-settings/addon.js"
import "./chart-display/addon.js"
