""" Versioning information for the GDS

This file contains versioning and compatibility definitions for the GDS tool set.
"""

# Currently the GDS is backwards compatible with all dictionaries. This MUST be bumped with each framework release to
# ensure compatibility
MINIMUM_SUPPORTED_FRAMEWORK_VERSION = (0, 0, 0)
MAXIMUM_SUPPORTED_FRAMEWORK_VERSION = (3, 0, 0)
