/**
 * FP Backgrounds:
 *
 * F prime background colors are used to set the color of the various items in the system. This means highlighting
 * Events, and  Channels based on severity. These colors should define two pairs, the color, and a .table-hover hover
 * color for use in hover-tables.
 */
 .table-bordered th, .table-bordered td {
    border: 1px solid rgba(0, 0, 0, 0.125) !important;
}
.fp-color-fatal {
    color: #000000;
    background-color: rgba(251,128,114, 1);
}
.table-hover .fp-color-fatal:hover {
    color: #000000;
    background-color: rgba(251,128,114, 0.7);
}
.fp-color-warn-hi {
    color: #000000;
    background-color: rgba(242,142,44, 1);
}
.table-hover .fp-color-warn-hi:hover {
    color: #000000;
    background-color: rgb(242,142,44, 0.8);
}
.fp-color-warn-lo {
    color: #000000;
    background-color: rgba(237,201,73, 1);
}
.table-hover .fp-color-warn-lo:hover {
    color: #000000;
    background-color: rgba(237,201,73, 0.7);
}
.fp-color-act-hi {
    color: #000000;
    background-color: rgba(128,177,211, 1);
}
.table-hover .fp-color-act-hi:hover {
    color: #000000;
    background-color: rgba(128,177,211, 0.7);
}
.fp-color-act-lo {
    color: #000000;
    background-color: rgba(186,176,171, 1);
}
.table-hover .fp-color-act-lo:hover {
    color: #000000;
    background-color: rgba(186,176,171, 0.7);
}
.fp-color-command {
    color: #000000;
    background-color: rgba(127,201,127, 1);
}
.table-hover .fp-color-command:hover {
    color: #000000;
    background-color: rgba(127,201,127, 0.7);
}
.fp-color-logging {
    background-color: #f0fff0;
}
/**
 * Filter the orb image when there is no data flow.
 */
img.dead {
    -webkit-filter: grayscale(100%); /* Safari 6.0 - 9.0 */
    filter: grayscale(100%);
}

/**
 * FP Containerization:
 *
 * This section allows for providing flex and scroll capabilities. This allows our containers to fill the remaining
 * space without overflowing.  This is done by using "fp-full-height" on the top-level container. Then for each
 * non-growable items should be labeled with .fp-flex-header. Growable content's direct parent may use
 * "fp-scroll-container" and the content should be "fp-scrollable". "fp-flex-repeater" can label each intermediary
 * parent for the growable content which is not the direct parent.
 */
body, .fp-full-height {
    height: 100vh;
}
.fp-flex-container {
    display: flex;
    flex-direction: column;
}
.fp-flex-repeater {
    flex-grow: 1;
    display: flex;
    flex-direction: column;
}
.fp-flex-header {
    flex-grow: 0;
}
.fp-scroll-container {
    overflow: hidden;
    flex-grow: 1;
    position: relative;
    min-height: 10em;
}
.fp-scrollable {
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    overflow: auto;
}


/**
 * Based on work from: https://www.abeautifulsite.net/whipping-file-inputs-into-shape-with-bootstrap-3 these CSS items
 * help make the file input looks like other buttons. Thus allowing it to play nicely with the other bootstrap buttons
 * being used by the project.
 */
.btn-file {
    position: relative;
    overflow: hidden;
    height: 100%;
}
/**
 * FP scrollable container. This will allow a container to sub-scroll rather than increase the page's size. This will
 * allow the user to continue to use the page controls without worry to hiding the controls at the top.
 */
.fp-scroll-container {
    overflow: auto;
}

/*https://www.w3schools.com/css/css_tooltip.asp*/
.tooltip {
    position: relative;
    display: inline-block;
    border-bottom: 1px dotted black;
}

.tooltip .tooltiptext {
    visibility: hidden;
    width: 120px;
    background-color: black;
    color: #fff;
    text-align: center;
    border-radius: 6px;
    padding: 5px 0;

    /* Position the tooltip */
    position: absolute;
    z-index: 1;
}

.tooltip:hover .tooltiptext {
    visibility: visible;
}


/**
 * Table CSS to allow for the last child to take up its needed space. This means the last column of the table will be
 * displayed taking up the full amount of the remaining space.
 */
table {
    width: auto;
    white-space: nowrap;
    text-overflow: ellipsis;
}
table td:last-child {
    width: 100%;
}


/**
 * Dashboard CSS Styles for styling components specific to the Dashboard page
 */
.fp-dashboard-box {
    margin: 2px;
    padding: 5px;
    border: 1px solid black;
    border-radius: 5px;
}
.fp-dashboard-box-title {
    width: initial;
    color: white;
    background: black;
    border-radius: 5px;
}

/**
* Style related to Events page
**/
.fp-auto-increment {
    background-color: #28a745 !important;
    color: #fff !important;
}

/**
* Style related to Charts page
**/
.fp-chart-btn-icon {
    font-size: 1.5rem; 
    font-weight: 700; 
    line-height: 1; 
    color: #fff;
}

.fp-chart-btn-text {
    font-size: 1rem; 
    font-weight: 700; 
    line-height: 2; 
    color: #fff;
}

.fp-resize-box {
    resize: vertical; 
    overflow: auto; 
    padding: 1em;
}

.fade-enter-active, .fade-leave-active {
    transition: opacity .5s;
}

.fade-enter, .fade-leave-to {
    opacity: 0;
}

.click-able-item {
    cursor: pointer !important;
}

.navbar-custom-style-blue-whale {
    background-color: rgba(17, 23, 47, 1) !important; 
}

.navbar-custom-style-purple {
    background-color: rgba(119, 41, 83, 1) !important; 
}

.navbar-custom-style-dark-blue {
    background-color: rgba(55, 90, 127, 1) !important; 
}

.navbar-custom-style-dark-gray {
    background-color: rgba(48, 48, 48, 1) !important; 
}

.navbar-custom-style-indigo {
    background-color: rgba(89, 49, 150, 1) !important; 
}

.navbar-custom-style-sapphire {
    background-color: rgba(7, 54, 66, 1) !important; 
}

.navbar-custom-style-lotus {
    background-color: rgba(143, 83, 78, 1) !important; 
}

.navbar-custom-style-red-oxide {
    background-color: rgba(75, 20, 22, 1) !important; 
}
